@@ -132,16 +132,11 @@ static GType _gst_buffer_type = 0;
 void
 _gst_buffer_initialize (void)
 {
-  gpointer ptr;
-
-  gst_buffer_get_type ();
-  gst_subbuffer_get_type ();
-
   /* the GstMiniObject types need to be class_ref'd once before it can be
    * done from multiple threads;
    * see http://bugzilla.gnome.org/show_bug.cgi?id=304551 */
-  ptr = g_type_class_ref (GST_TYPE_BUFFER);
-  g_type_class_unref (ptr);
+  g_type_class_ref (gst_buffer_get_type ());
+  g_type_class_ref (gst_subbuffer_get_type ());
 }
 
 GType
@@ -385,8 +380,9 @@ gst_buffer_try_new_and_alloc (guint size)
  * Returns: a reference to the #GstCaps. unref after usage.
  * Returns NULL if there were no caps on this buffer.
  */
-/* FIXME can we make this threadsafe without a lock on the buffer?
- * We can use compare and swap and atomic reads. */
+/* this is not made atomic because if the buffer were reffed from multiple
+ * threads, it would have a refcount > 2 and thus be immutable.
+ */
 GstCaps *
 gst_buffer_get_caps (GstBuffer * buffer)
 {
@@ -411,10 +407,9 @@ gst_buffer_get_caps (GstBuffer * buffer)
  * be increased and any previous caps on the buffer will be
  * unreffed.
  */
-/* FIXME can we make this threadsafe without a lock on the buffer?
- * We can use compare and swap and atomic reads. Another idea is to
- * not attach the caps to the buffer but use an event to signal a caps
- * change. */
+/* this is not made atomic because if the buffer were reffed from multiple
+ * threads, it would have a refcount > 2 and thus be immutable.
+ */
 void
 gst_buffer_set_caps (GstBuffer * buffer, GstCaps * caps)
 {
